; 
    .code
; 
; This procedure takes a source address and a byte count, allocates a
; buffer, builds a hex-ascii dump of the source memory in the buffer,
; and returns the address of the buffer, or zero if the byte count is
; zero or the allocation failed. The source memory must lie within
; the address space of the calling process and be readable.
; 
HexAsciiDump proc pSrc:DWORD, cbSrc:DWORD

    LOCAL cbExtra :DWORD
    LOCAL pDest   :DWORD

    push ebx
    push esi
    push edi

    .IF cbSrc == 0
      return 0
    .ENDIF

    mov esi, pSrc

    mov eax, cbSrc
    mov edx, eax
    shr eax, 4          ; calc paragraphs
    mov ebx, eax
    and edx, 15         ; calc bytes in partial paragraph
    mov cbExtra, edx
    .IF edx
      inc eax           ; allow full paragraph for partial
    .ENDIF
    mov ecx, 79
    mul ecx             ; calc buffer size
    inc eax             ; allow for terminating null

    push eax
    mov edi, halloc(eax)
    mov pDest, edi
    pop eax

    .IF edi == 0
      return 0
    .ENDIF

    invoke memfill, edi, eax, 20202020h

    mov ecx, ebx
    test ecx, ecx
    jz  partial_paragraph
    jmp paragraph_loop

    align 16

  hex_table:

    db "000102030405060708090A0B0C0D0E0F"
    db "101112131415161718191A1B1C1D1E1F"
    db "202122232425262728292A2B2C2D2E2F"
    db "303132333435363738393A3B3C3D3E3F"
    db "404142434445464748494A4B4C4D4E4F"
    db "505152535455565758595A5B5C5D5E5F"
    db "606162636465666768696A6B6C6D6E6F"
    db "707172737475767778797A7B7C7D7E7F"
    db "808182838485868788898A8B8C8D8E8F"
    db "909192939495969798999A9B9C9D9E9F"
    db "A0A1A2A3A4A5A6A7A8A9AAABACADAEAF"
    db "B0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF"
    db "C0C1C2C3C4C5C6C7C8C9CACBCCCDCECF"
    db "D0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF"
    db "E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEF"
    db "F0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF"

  ascii_table:

    db "................"
    db "................"
    db ' !"',"#$%&'()*+,-./"
    db "0123456789:;<=>?"
    db "@ABCDEFGHIJKLMNO"
    db "PQRSTUVWXYZ[\]^_"
    db "`abcdefghijklmno"
    db "pqrstuvwxyz{|}~."
    db "................"
    db "................"
    db "................"
    db "................"
    db "................"
    db "................"
    db "................"
    db "................"

  hex_address:

    push ecx
    mov ecx, esi
    movzx edx, cl
    movzx eax, WORD PTR [ebx+edx*2]
    mov [edi+6], ax
    shr ecx, 8
    movzx edx, cl
    movzx eax, WORD PTR [ebx+edx*2]
    mov [edi+4], ax
    shr ecx, 8
    movzx edx, cl
    movzx eax, WORD PTR [ebx+edx*2]
    mov [edi+2], ax
    shr ecx, 8
    movzx edx, cl
    movzx eax, WORD PTR [ebx+edx*2]
    mov [edi], ax
    add edi, 8
    pop ecx
    retn

    align 16

  paragraph_loop:

    lea ebx, hex_table
    call hex_address
    add edi, 1

    push esi

    REPEAT 8
      movzx edx, BYTE PTR [esi]
      movzx eax, WORD PTR [ebx+edx*2]
      mov [edi+1], ax
      add esi, 1
      add edi, 3
    ENDM

    mov BYTE PTR [edi+1], 2Dh
    add edi, 2

    REPEAT 8
      movzx edx, BYTE PTR [esi]
      movzx eax, WORD PTR [ebx+edx*2]
      mov [edi+1], ax
      add esi, 1
      add edi, 3
    ENDM

    pop esi
    add edi, 2

    lea ebx, ascii_table
    REPEAT 16
      movzx edx, BYTE PTR [esi]
      movzx eax, BYTE PTR [ebx+edx]
      mov [edi], al
      add esi, 1
      add edi, 1
    ENDM

    mov WORD PTR [edi], 0A0Dh
    add edi, 2

    sub ecx, 1
    jnz paragraph_loop

  partial_paragraph:

    mov ecx, cbExtra
    test ecx, ecx
    jz  finished

    lea ebx, hex_table
    call hex_address

    add edi, 1

    push ecx
    push esi

    mov edx, 8

  hex_loop1:

    sub ecx, 1
    js  @F
    movzx eax, BYTE PTR [esi]
    movzx eax, WORD PTR [ebx+eax*2]
    mov [edi+1], ax
  @@:
    add edi, 3
    add esi, 1

    sub edx, 1
    jnz hex_loop1

    cmp ecx, 1
    jl  @F
    mov BYTE PTR [edi+1], 2Dh
  @@:
    add edi, 2

    mov edx, 8

  hex_loop2:

    sub ecx, 1
    js  @F
    movzx eax, BYTE PTR [esi]
    movzx eax, WORD PTR [ebx+eax*2]
    mov [edi+1], ax
  @@:
    add edi, 3
    add esi, 1

    sub edx, 1
    jnz hex_loop2

    pop esi
    pop ecx

    add edi, 2

    lea ebx, ascii_table
    mov edx, 16

  ascii_loop:

    sub ecx, 1
    js @F
    movzx eax, BYTE PTR [esi]
    movzx eax, BYTE PTR [ebx+eax]
    mov [edi], al
  @@:
    add esi, 1
    add edi, 1

    sub edx, 1
    jnz ascii_loop

    mov WORD PTR [edi], 0A0Dh

  finished:

    mov BYTE PTR [edi+2], 0

    pop edi
    pop esi
    pop ebx

    return pDest

HexAsciiDump endp

; 
